﻿using Ant_Colony_Optimalization_Parallel.Graph;
using Ant_Colony_Optimalization_Parallel.Other;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization_Parallel.Ant
{
   

    public class CAntCompleteGraph : IComparable<CAntCompleteGraph>
    {
        private int visitedNodesCount;
        private double alpha;
        private double beta;
        private int nodeEdgeCount;
        private int nodeCount;
        public int[] VisitedEdgesIndex { get; set; }
        public CEdge[] VisitedEdges { get; set; }

        private int stepNumber;
        public HashSet<CNode> VisitedNodes { get; set; }

        public bool[] VisitedNodes2;
        CNode ActualNode { get; set; }
        CNode StartingNode { get; set; }

        private IPheromoneIncrease PheromonIncreaseFunction { get; set; }
        public bool PathFound { get; set; }

        public double Distance { get; set; }
        public CAntCompleteGraph( CNode startingNode, int nodeCount, double alpha, double beta, double max)
        {
            this.stepNumber = 1;
            this.nodeCount = nodeCount;
            this.nodeEdgeCount = startingNode.edgeCount;
            this.visitedNodesCount = 1;
            this.VisitedEdges = new CEdge[nodeCount];
            this.VisitedNodes = new HashSet<CNode>();
            this.VisitedEdgesIndex = new int[nodeCount];
            this.VisitedNodes.Add(startingNode);
            this.ActualNode = startingNode;
            this.StartingNode = startingNode;
            this.Distance = 0;
            this.alpha = alpha;
            this.beta = beta;
            this.PheromonIncreaseFunction = new CIncreaseConstant(max);
            this.VisitedNodes2 = new bool[nodeCount];
            this.VisitedNodes2[startingNode.id] = true;
        }

        public void Reset(){
            this.stepNumber = 1;
            this.visitedNodesCount = 1;
            this.VisitedEdges = new CEdge[this.nodeCount];
            this.VisitedNodes = new HashSet<CNode>();
            this.VisitedEdgesIndex = new int[nodeCount];
            this.VisitedNodes.Add(StartingNode);
            this.ActualNode = StartingNode;
            this.Distance = 0;
            this.VisitedNodes2 = new bool[nodeCount];
            this.VisitedNodes2[StartingNode.id] = true;
        }

        public bool MoveToNextNode()
        {
            Tuple<int, CEdge>[] notVisitedEdgesArray = this.GetEdgesToNotVisitedNodes();
            if (notVisitedEdgesArray.Count() == 0)
            {
                this.MoveToStart();
                return false;
            }
            else
            {
                int edgeIndex = this.ChooseNextNodeIndex2(notVisitedEdgesArray);
                CEdge choosenEdge = this.GetEdgeByIndex(edgeIndex);
                this.Distance += choosenEdge.Distance;
                CNode choosenNode = this.GetNeighboringNodeOnEdge(choosenEdge);
                this.VisitedEdges[this.stepNumber - 1] = choosenEdge;
                this.VisitedEdgesIndex[this.stepNumber - 1] = edgeIndex;
                this.VisitedNodes.Add(choosenNode);
                this.VisitedNodes2[choosenNode.id] = true;
                this.stepNumber++;
                this.ActualNode = choosenNode;
                this.visitedNodesCount++;
                return true;
            }
        }

        public void MoveToStart()
        {
            int counter = 0;
            while (!((this.ActualNode.EdgeArray[counter].GetNeighbor(this.ActualNode.id)).Equals(this.StartingNode)))
            {
                counter++;
            }
            CEdge choosenEdge = this.GetEdgeByIndex(counter);
            this.Distance += choosenEdge.Distance;
            // Console.WriteLine("index hrany : " + counter.ToString());
            // Console.WriteLine("hrana " + choosenEdge.Id);
            CNode choosenNode = this.GetNeighboringNodeOnEdge(choosenEdge);
            // Console.WriteLine("uzel " + choosenNode.id);
            this.VisitedEdges[this.stepNumber - 1] = choosenEdge;
            this.VisitedEdgesIndex[this.stepNumber - 1] = counter;
            this.VisitedNodes.Add(choosenNode);
            this.stepNumber++;
            this.ActualNode = choosenNode;
            this.visitedNodesCount++;
        }


        private Tuple<int, CEdge>[] GetEdgesToNotVisitedNodes()
        {
            CEdge[] nodeEdgesArray = this.ActualNode.EdgeArray;
            CNode[] neighborNodeIdArray = this.ActualNode.NeighborNodeIdArray;
            int maxCount = this.nodeEdgeCount - this.visitedNodesCount + 1;
            Tuple<int, CEdge>[] notVisitedEdgesArray = new Tuple<int, CEdge>[maxCount];
            int notVisitedNodesCounter = 0;

            int arrayCount = neighborNodeIdArray.Count();
            for (int a = 0; a < arrayCount &&  notVisitedNodesCounter<maxCount; a++)
            {
                CNode nodeNeighbor = neighborNodeIdArray[a];
                //if (this.VisitedNodes.Contains(nodeNeighbor))
                if (this.VisitedNodes2[nodeNeighbor.id] == true)
                {

                }
                else
                {
                    notVisitedEdgesArray[notVisitedNodesCounter] = new Tuple<int, CEdge>(a, nodeEdgesArray[a]);
                    notVisitedNodesCounter++;
                }
            }
            return notVisitedEdgesArray;
        }

        private int ChooseNextNodeIndex2(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            if (notVisitedEdgesArray.Count() == 1)
            {
                return notVisitedEdgesArray[0].Item1;
            }
            else
            {
                double[] probabilityArray = this.RecalculateProbabilityArray(notVisitedEdgesArray);
                double[] cumProbabilityArray = this.RecalculateCumulProbabilityArray(probabilityArray);

                double randomNumber = CRandomProvider.GetThreadRandom().NextDouble();
                // double randomNumber = this.GetRandomNumber(0, 1);
                int counter = 0;

                while (randomNumber >= cumProbabilityArray[counter])
                {
                    counter++;
                }
                return notVisitedEdgesArray[counter].Item1;
            }
        }

        private double[] RecalculateProbabilityArray(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            double[] probabilityArray = new double[notVisitedEdgesArray.Count()];

            int count = notVisitedEdgesArray.Count();
            for (int i = 0; i < count; i++)
            {
                CEdge edge = notVisitedEdgesArray[i].Item2;
                if (edge.PheromoneChange())
                {
                    double taueta = this.CalculateTauete(edge.Pheromone, edge.Distance);
                    probabilityArray[i] = taueta;
                    edge.TeuteOld = taueta;
                }
                else
                {
                    probabilityArray[i] = edge.TeuteOld;
                }
            }
            return probabilityArray;
        }
        public double[] RecalculateCumulProbabilityArray(double[] probabilityArray)
        {
            double[] cumulProbabilityArray = new double[probabilityArray.Count()];
            double arraySum = probabilityArray.Sum();


            cumulProbabilityArray[0] = probabilityArray[0] / arraySum;
            int count = probabilityArray.Count();
            for (int i = 1; i < count; i++)
            {
                double i1 = cumulProbabilityArray[i - 1];
                double i2 = probabilityArray[i] / arraySum;
                cumulProbabilityArray[i] = i1 + i2;
            }
            cumulProbabilityArray[cumulProbabilityArray.Count() - 1] = 1;

            return cumulProbabilityArray;
        }

        private CEdge GetEdgeByIndex(int index)
        {
            return this.ActualNode.EdgeArray[index];
        }

        private CNode GetNeighboringNodeOnEdge(CEdge edge)
        {
            if (edge.Node1 == (this.ActualNode))
            {
                return edge.Node2;
            }
            else
            {
                return edge.Node1;
            }
        }

        public double CalculateTauete(double pheromone, double distance)
        {
            double tauete = System.Math.Pow(pheromone, this.alpha) * System.Math.Pow((1 / distance), this.beta);
            return tauete;
        }

        public void ActualizePheromone()
        {
            this.PheromonIncreaseFunction.IncreasePheromone(this.VisitedEdges);
        }


        public int CompareTo(object obj)
        {
            return this.Distance.CompareTo(((CAntCompleteGraph)obj).Distance);
        }


        public int CompareTo(CAntCompleteGraph other)
        {
            return this.Distance.CompareTo(other.Distance);
        }
    }
}
