﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization_Parallel.Graph
{
    public class CEdge
    {
        double pheromone;
        double pheromoneOld;
        double teuteOld;
        public double Id { get; set; }
        public double Pheromone
        {
            get { return this.pheromone; }
            set
            {
                this.pheromoneOld = this.pheromone;
                this.pheromone = value;
            }
        }

        public double TeuteOld
        {
            get { return this.teuteOld; }
            set
            {
                this.pheromoneOld = this.pheromone;
                this.teuteOld = value;
            }
        }
        public double Distance { get; set; }
        public CNode Node1 { get; set; }
        public CNode Node2 { get; set; }

        private Random random = new Random();

        public CEdge(int id, CNode node1, CNode node2)
        {
            this.Id = id;
            node1.AddEdge(this);
            node2.AddEdge(this);
            this.Node1 = node1;
            this.Node2 = node2;
            this.Pheromone = 0.5;
            this.pheromoneOld = 0.5;
            this.Distance = this.CalculateDistance(node1, node2);
        }

        public CNode GetNeighbor(int id)
        {
            if (this.Node1.id == id)
            {
                return Node2;
            }
            else
            {
                return Node1;
            }
        }

        private double CalculateDistance(CNode node1, CNode node2)
        {
            double xDiff = node1.Point.X - node2.Point.X;
            double yDiff = node1.Point.Y - node2.Point.Y;

            double x2 = xDiff * xDiff;
            double y2 = yDiff * yDiff;

            double c2 = x2 + y2;
            double result = Math.Round(Math.Sqrt(c2), 3);
            return result;
        }

        internal void Destroy()
        {
            this.Node1.RemoveEdge(this);
            this.Node2.RemoveEdge(this);
        }

        public int CompareTo(object obj)
        {
            return this.Distance.CompareTo(((CEdge)obj).Distance);
        }

        public string ToStringExport()
        {
            return "EDGE_RESULT:[" + this.Node1.ToStringExport() + "-" + this.Node2.ToStringExport() + "]";
        }

        public bool PheromoneChange()
        {
            if (this.pheromone == this.pheromoneOld)
            {
                return false;
            }
            else
            {
                return true;
            }
        }
    }
}
