﻿using Ant_Colony_Optimalization.Ant.PheromoneActualization;
using Ant_Colony_Optimalization.Generator;
using Ant_Colony_Optimalization.Graph;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization.Ant
{
    public class CAntCompleteGraph : IAnt, IComparable
    {
        int id;

        // public double[] ProbabilityArray { get; set; }
        // public double[] CumulProbabilityArray { get; set; }

        // public LinkedList<CVisiterRoutePoint> VisitedNodes { get; set; }
        public int[] VisitedEdgesIndex { get; set; }
        public CEdge[] VisitedEdges { get; set; }

        private int stepNumber = 1;
        public CNode[] VisitedNodes { get; set; }
        CNode ActualNode { get; set; }
        CNode StartingNode { get; set; }
        CNode EndingNode { get; set; }
        private int visitedNodesCount;
        private Random random;
        private double alpha;
        private double beta;
        private IPheromoneIncrease PheromonIncreaseFunction { get; set; }
        public bool PathFound { get; set; }

        public double Distance { get; set; }
        public CAntCompleteGraph(int id,CNode startingNode, int nodeCount, double alpha, double beta, IPheromoneIncrease pheromonIncreaseFunction)
        {
            this.id = id;
            this.visitedNodesCount = 1;
            this.VisitedEdges = new CEdge[nodeCount];
            this.VisitedNodes = new CNode[nodeCount + 1];
            this.VisitedEdgesIndex = new int[nodeCount];
            this.VisitedNodes[0] = startingNode;
            this.ActualNode = startingNode;
            this.StartingNode = startingNode;
            this.random = new Random();
            this.Distance = 0;
            this.alpha = alpha;
            this.beta = beta;
            this.PheromonIncreaseFunction = pheromonIncreaseFunction;
        }

        public bool MoveToNextNode()
        {

            Tuple<int, CEdge>[] notVisitedEdgesArray = this.GetEdgesToNotVisitedNodes();
            if (notVisitedEdgesArray.Count() == 0)
            {
                this.MoveToStart();
                return false;
            }
            else
            {
                int edgeIndex = this.ChooseNextNodeIndex(notVisitedEdgesArray);
                CEdge choosenEdge = this.GetEdgeByIndex(edgeIndex);
                this.Distance += choosenEdge.Distance;
                CNode choosenNode = this.GetNeighboringNodeOnEdge(choosenEdge);
                this.VisitedEdges[this.stepNumber - 1] = choosenEdge;
                this.VisitedEdgesIndex[this.stepNumber - 1] = edgeIndex;
                this.VisitedNodes[this.stepNumber] = choosenNode;
                this.stepNumber++;
                this.ActualNode = choosenNode;
                this.visitedNodesCount++;
                return true;
            }
        }

        public void MoveToStart()
        {
            int counter = 0;
            while (!((this.ActualNode.EdgeArray[counter].GetNeighbor(this.ActualNode)).Equals(this.StartingNode)))
            {
                counter++;
            }
            CEdge choosenEdge = this.GetEdgeByIndex(counter);
            this.Distance += choosenEdge.Distance;
            // Console.WriteLine("index hrany : " + counter.ToString());
            // Console.WriteLine("hrana " + choosenEdge.Id);
            CNode choosenNode = this.GetNeighboringNodeOnEdge(choosenEdge);
            // Console.WriteLine("uzel " + choosenNode.id);
            this.VisitedEdges[this.stepNumber - 1] = choosenEdge;
            this.VisitedEdgesIndex[this.stepNumber - 1] = counter;
            this.VisitedNodes[this.stepNumber] = choosenNode;
            this.stepNumber++;
            this.ActualNode = choosenNode;
            this.visitedNodesCount++;
        }

        private Tuple<int, CEdge>[] GetEdgesToNotVisitedNodes()
        {
            CEdge[] nodeEdgesArray = this.ActualNode.EdgeArray;
            Tuple<int, CEdge>[] notVisitedEdgesArray = new Tuple<int, CEdge>[nodeEdgesArray.Count() - this.visitedNodesCount + 1];
            int notVisitedNodesCounter = 0;

            for (int a = 0; a < nodeEdgesArray.Count(); a++)
            {
                CNode nodeNeighbor = nodeEdgesArray[a].GetNeighbor(this.ActualNode);
                if (this.VisitedNodes.Contains(nodeNeighbor))
                {

                }
                else
                {
                    notVisitedEdgesArray[notVisitedNodesCounter] = new Tuple<int, CEdge>(a, nodeEdgesArray[a]);
                    notVisitedNodesCounter++;
                }
            }
            return notVisitedEdgesArray;
        }

        private int ChooseNextNodeIndex(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            if (notVisitedEdgesArray.Count() == 1)
            {
                return notVisitedEdgesArray[0].Item1;
            }
            else
            {
                double[] probabilityArray = this.RecalculateProbabilityArray(notVisitedEdgesArray);
                double[] cumProbabilityArray = this.RecalculateCumulProbabilityArray(probabilityArray);

                double randomNumber = this.random.NextDouble();
                // double randomNumber = this.GetRandomNumber(0, 1);
                int counter = 0;

                while (randomNumber >= cumProbabilityArray[counter])
                {
                    counter++;
                }
                return notVisitedEdgesArray[counter].Item1;
            }
        }

        
        private int ChooseNextNodeIndexNearestNeighbor(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            if (notVisitedEdgesArray.Count() == 1)
            {
                return notVisitedEdgesArray[0].Item1;
            }
            else
            {
                Tuple<CEdge, int>[] notVisitedEdgesArray2 = new Tuple<CEdge, int>[notVisitedEdgesArray.Length];
                for (int i = 0; i < notVisitedEdgesArray.Count(); i++)
                {
                    notVisitedEdgesArray2[i] = new Tuple<CEdge, int>(notVisitedEdgesArray[i].Item2, notVisitedEdgesArray[i].Item1);
                }
                Array.Sort(notVisitedEdgesArray2);
                return notVisitedEdgesArray2[0].Item2;
            }
        }

        private double[] RecalculateProbabilityArray(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            double[] probabilityArray = new double[notVisitedEdgesArray.Count()];

            for (int i = 0; i < notVisitedEdgesArray.Count(); i++)
            {
                CEdge edge = notVisitedEdgesArray[i].Item2;
                probabilityArray[i] = this.CalculateTauete(edge.Pheromone, edge.Distance);
            }
            return probabilityArray;
        }
        public double[] RecalculateCumulProbabilityArray(double[] probabilityArray)
        {
            double[] cumulProbabilityArray = new double[probabilityArray.Count()];
            double arraySum = probabilityArray.Sum();


            cumulProbabilityArray[0] = probabilityArray[0] / arraySum;
            for (int i = 1; i < probabilityArray.Count(); i++)
            {
                double i1 = cumulProbabilityArray[i - 1];
                double i2 = probabilityArray[i] / arraySum;
                cumulProbabilityArray[i] = i1 + i2;
            }
            cumulProbabilityArray[cumulProbabilityArray.Count() - 1] = 1;

            return cumulProbabilityArray;
        }

        private CEdge GetEdgeByIndex(int index)
        {
            return this.ActualNode.EdgeArray[index];
        }

        private CNode GetNeighboringNodeOnEdge(CEdge edge)
        {
            if (edge.Node1 == (this.ActualNode))
            {
                return edge.Node2;
            }
            else
            {
                return edge.Node1;
            }
        }

        public double CalculateTauete(double pheromone, double distance)
        {
            double tauete = System.Math.Pow(pheromone, this.alpha) * System.Math.Pow((1 / distance), this.beta);
            return tauete;
        }

        public void ActualizePheromone()
        {
            this.PheromonIncreaseFunction.IncreasePheromone(this.VisitedEdges);
        }


        public int CompareTo(object obj)
        {
            return this.Distance.CompareTo(((CAntCompleteGraph)obj).Distance);
        }

    }
}
