﻿using Ant_Colony_Optimalization.Graphic.Object;
using System;

namespace Ant_Colony_Optimalization.Graph
{
    public class CEdge : IComparable
    {
        public double Id { get; set; }
        public double Pheromone { get; set; }

        public double Distance { get; set; }
        public CNode Node1 { get; set; }
        public CNode Node2 { get; set; }

        private Random random = new Random();

        public CEdge(int id, CNode node1, CNode node2)
        {
            this.Id = id;
            node1.AddEdge(this);
            node2.AddEdge(this);
            this.Node1 = node1;
            this.Node2 = node2;
            this.Pheromone = 0.5;
            this.Distance = this.CalculateDistance(node1, node2);
        }

        public CEdge(int id, CNode node1, CNode node2, double distance)
        {
            this.Id = id;
            node1.AddEdge(this);
            node2.AddEdge(this);
            this.Node1 = node1;
            this.Node2 = node2;
            this.Pheromone = 0.5;
            this.Distance = distance;
        }
        public double GetRandomNumber(double minimum, double maximum)
        {
            return this.Id + 1;
            //return random.NextDouble() * (maximum - minimum) + minimum;
        }

        public CNode GetNeighbor(CNode node)
        {
            if (this.Node1.Equals(node))
            {
                return Node2;
            }
            else
            {
                return Node1;
            }
        }

        private double CalculateDistance(CNode node1, CNode node2)
        {
            double xDiff = node1.Point.X - node2.Point.X;
            double yDiff = node1.Point.Y - node2.Point.Y;

            double x2 = xDiff * xDiff;
            double y2 = yDiff * yDiff;

            double c2 = x2 + y2;
            double result = Math.Round(Math.Sqrt(c2), 3);
            return result;
        }

        public CEdgeLine ToCEdgeLine()
        {
            return new CEdgeLine(this.Node1.Point, this.Node2.Point);
        }

        internal void Destroy()
        {
            this.Node1.RemoveEdge(this);
            this.Node2.RemoveEdge(this);
        }

        public int CompareTo(object obj)
        {
            return this.Distance.CompareTo(((CEdge)obj).Distance);
        }
    }
}
