﻿using Ant_Colony_Optimalization.Graph;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Ant_Colony_Optimalization.Graphic.Render
{
    public class CNodeRender
    {
        private Panel panel;
        private Pen pen;
        private SolidBrush sb;

        public CNodeRender(Panel panel)
        {
            this.panel = panel;
        }

        public void Draw(IEnumerable nodeList)
        {
            try
            {
                Graphics g = this.panel.CreateGraphics();
                Font drawFont = new System.Drawing.Font("Arial", 10);
                SolidBrush drawBrush = new SolidBrush(System.Drawing.Color.Black);
                StringFormat drawFormat = new System.Drawing.StringFormat();
                this.pen = new Pen(Color.Red);
                this.sb = new SolidBrush(Color.Red);
                int counter = 0;
                foreach (CNode p in nodeList)
                {
                    int x = p.Point.X;
                    int y = p.Point.Y;
                    g.DrawEllipse(pen, x - 5, y - 5, 10, 10);
                    g.FillEllipse(sb, x - 5, y - 5, 10, 10);
                    g.DrawString(counter.ToString(), drawFont, drawBrush, x + 5, y + 5, drawFormat);

                    counter++;
                }
                this.pen.Dispose();
                this.sb.Dispose();
                drawFont.Dispose();
                drawBrush.Dispose();

                g.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        internal void DrawPheromone(LinkedList<CNodePhoremoneStrenge> linkedList, int width, int height)
        {
            try
            {
                Graphics g = this.panel.CreateGraphics();
                Color myColor;

                foreach (CNodePhoremoneStrenge p in linkedList)
                {
                    {
                        myColor = Color.FromArgb(255 - p.StrengeColorValue, 255 - p.StrengeColorValue, 255);
                        this.sb = new SolidBrush(myColor);
                        int x = p.Point.X;
                        int y = p.Point.Y;
                        g.FillRectangle(sb, x - width / 2, y - height / 2, width, height);
                        this.sb.Dispose();
                    }
                }


                g.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private double CalculateDistance(CNode node1, CNode node2)
        {
            double xDiff = node1.Point.X - node2.Point.X;
            double yDiff = node1.Point.Y - node2.Point.Y;

            double x2 = xDiff * xDiff;
            double y2 = yDiff * yDiff;

            double c2 = x2 + y2;
            double result = Math.Round(Math.Sqrt(c2), 3);
            return result;
        }
    }
}
